/*
 * Copyright 2002 Barnaba Marcello <vjt@azzurra.org>
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2a. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 2b. Redistribution in binary form requires specific prior written
 *     authorization of the maintainer.
 * 
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *    This product includes software developed by Barnaba Marcello.
 * 
 * 4. The names of the maintainer, developers and contributors may not be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE MAINTAINER, DEVELOPERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * THE DEVELOPERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/* $Id: agent-ep.h,v 1.7 2003/03/23 18:50:38 vjt Exp $ */

#ifndef __agent_ep_h__
#define __agent_ep_h__

#include "parse.h"

#define AGENT_ENTRY_POINT(NAME, STR_NAME, USER_PTR, HASH_FUNC)	\
	gboolean G_INLINE_FUNC (NAME)(User *caller, gchar *str)	\
	{							\
	    const struct STR_NAME *c;				\
	    static gchar *parv[MAXPARA];			\
	    gint parc;						\
	    extern User * USER_PTR;				\
	    register gchar *s;					\
	    gint len;						\
	    							\
	    s = strchr(str, ' ');				\
	    if(s)						\
	    {							\
		*s = '\0';					\
		len = s++ - str;				\
	    }							\
	    else						\
	    {							\
		len = strlen(str);				\
	    }							\
								\
	    my_strtoupper(str);					\
								\
	    if((c = HASH_FUNC (str, len)))			\
	    {							\
		if(c->para && s)				\
		{						\
		    /* command with params			\
		     */						\
		    memset((gpointer)parv, 0x0, 		\
			    sizeof(gchar *) * MAXPARA);		\
		    my_g_strsplit(s, ' ', c->para, &parc, parv);\
		    (*c->func) (caller, parc, parv);		\
		}						\
		else						\
		{						\
		    /* command without params			\
		     */						\
		    (*c->func) (caller, 0, NULL);		\
		}						\
								\
		return TRUE;					\
	    }							\
								\
	    /* unknown command ..				\
	     */							\
	    send_out(":%s NOTICE %s :Unknown command, "		\
		    "type /MSG %s HELP for a list "		\
		    "of available commands",			\
		    (USER_PTR)->name, caller->name,		\
		    (USER_PTR)->name);				\
	    							\
	    return TRUE;					\
	}

#endif
