/*
 * Copyright 2002 Barnaba Marcello <vjt@azzurra.org>
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2a. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 2b. Redistribution in binary form requires specific prior written
 *     authorization of the maintainer.
 * 
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *    This product includes software developed by Barnaba Marcello.
 * 
 * 4. The names of the maintainer, developers and contributors may not be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE MAINTAINER, DEVELOPERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * THE DEVELOPERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/* $Id: sql.h,v 1.20 2003/03/26 21:17:48 vjt Exp $ */

#ifndef __sql_h__
#define __sql_h__

#include <mysql.h>
#include "log.h"

gboolean sql_connect(void);
void sql_shutdown(void);
void sql_thread_init(void);
void sql_cleanup_before_exit(void);
G_INLINE_FUNC void sql_lock_tables(void);
G_INLINE_FUNC void sql_unlock_tables(void);

G_INLINE_FUNC gboolean __sql_query_assert(MYSQL_RES **, const gchar *, ...) G_GNUC_PRINTF(2, 3);
G_INLINE_FUNC glong sql_affected_rows(void);
G_INLINE_FUNC guint sql_insert_id(void);

G_INLINE_FUNC glong sql_thread_id(void);
G_INLINE_FUNC guint sql_get_proto_info(void);
G_INLINE_FUNC gchar *sql_get_server_info(void);
G_INLINE_FUNC gchar *sql_get_host_info(void);

#ifdef G_HAVE_ISO_VARARGS
#define sql_query_assert(rs, q ...)			\
	if(!__sql_query_assert(rs, q))			\
	    return;
#define sql_query_assert_ret(ret, rs, q ...)		\
	if(!__sql_query_assert(rs, q))			\
	    return(ret);

#elif defined(G_HAVE_GNUC_VARARGS) /* G_HAVE_ISO_VARARGS */
#define sql_query_assert(rs, q, ...)			\
	if(!__sql_query_assert(rs, q, ##__VA_ARGS__))	\
	    return;
#define sql_query_assert_ret(ret, rs, q, ...)		\
	if(!__sql_query_assert(rs, q, ##__VA_ARGS__))	\
	    return(ret);

#else /* defined(G_HAVE_GNUC_VARARGS) */
#error "GNUC_VARARGS or ISO_VARARGS are required !"
#endif

#define SQLMASKLEN 128

#endif
