/*
 * Copyright 2002 Barnaba Marcello <vjt@azzurra.org>
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2a. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 2b. Redistribution in binary form requires specific prior written
 *     authorization of the maintainer.
 * 
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *    This product includes software developed by Barnaba Marcello.
 * 
 * 4. The names of the maintainer, developers and contributors may not be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE MAINTAINER, DEVELOPERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * THE DEVELOPERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/* $Id: memoserv.c,v 1.26 2003/03/27 02:10:28 vjt Exp $ */

#include "sux.h"
#include "agents.h"
#include "memoserv.h"
#include "sql.h"
#include "network.h"
#include "main.h"
#include "help.h"
#include "parse.h"
#include "conf.h"
#include "modules.h"

User *ms;
static MYSQL_RES *ms_res;
static MYSQL_ROW ms_row;

REMOTE_TABLE_INSTANCE(user);

const gchar sux_module_name[] = "MemoServ";
const gchar sux_module_version[] = "0.1";
const gchar sux_module_info[] = "Memorandum Management Services";

static gboolean ms_memo_check(User *);
static G_INLINE_FUNC void memo_notify(User *, gushort);
static G_INLINE_FUNC gboolean ms_get_chan_and_founder_id(const gchar *, guint *, guint *);

guint ms_get_sendto_id(gchar *str);

/* for memoserv entry point, look into
 * memoserv-cmd.gperf
 */
gchar *g_module_check_init(GModule *mod)
{
    extern GSList *agents;
    Agent *agent;

    agent = g_new0(Agent, 1);
    agent->name = "MemoServ";
    agent->id = SYMBOL_SECTION_MEMOSERV;
    agent->u = &ms;
    agents = g_slist_prepend(agents, (gpointer) agent);

    agent_setcallback("MEMOSERV", memoserv);

    message_add_callback(MESSAGE_IDENTIFY, SUX_MSG_PRIORITY_LOW, (message_func) ms_memo_check);
    message_add_callback(MESSAGE_MSREQ, SUX_MSG_PRIORITY_NORMAL, (message_func) memoserv);

    if(me.boot)
    {
	agent_boot(agent);
    }

    g_message_syslog("MDL: Loaded MemoServ module");

    return NULL;
}

void g_module_unload(GModule *mod)
{
    extern GSList *agents;
    GSList *agent;

    agent = g_slist_find_custom(agents, GINT_TO_POINTER(SYMBOL_SECTION_MEMOSERV),
	    (GCompareFunc) agent_find);
    agents = g_slist_remove_link(agents, agent);
    agent_shutdown((Agent *)agent->data);

    g_free(agent->data);
    g_slist_free_1(agent);

    message_del_callback(MESSAGE_IDENTIFY, (message_func) ms_memo_check);
    message_del_callback(MESSAGE_MSREQ, (message_func) memoserv);

    g_message_syslog("MDL: Unloaded MemoServ module");
}

void ms_help(User *caller, gint parc, gchar **parv)
{
    sux_help(caller, ms, MEMOSERV_HELP_ID, parv ? parv[0] : NULL);
}

void ms_list(User *caller, gint parc, gchar **parv)
{
    gint i = 0;
    
    if(!(caller->mode & UMODE_r))
    {
	send_out(":%s NOTICE %s :You are not identified to nick %s !",
		ms->name, caller->name, caller->name);

	return;
    }

    sql_query_assert(&ms_res,
	    "SELECT `memo_sender`.`name`, DATE_FORMAT(`memo`.`date`, '%%b %%e %%Y - %%T'), `memo`.`read` "
	    "FROM `memo`, `nicks` `memo_sender`, `nicks` `memo_rcpt` "
	    "WHERE ( `memo`.`rcpt_id` = `memo_rcpt`.`id` AND `memo_rcpt`.`name` = '%s' ) AND"
	    	"( `memo`.`send_id` = `memo_sender`.`id` ) "
	    "ORDER BY `memo`.`date`",
	    caller->name);
    
    if(!mysql_num_rows(ms_res))
    {
	mysql_free_result(ms_res);

	send_out(":%s NOTICE %s :You have no memos",
		ms->name, caller->name);

	return;
    }
    
    send_out(":%s NOTICE %s :%ld memos found for %s. To read: /ms read NUMBER",
	    ms->name, caller->name, (glong) mysql_num_rows(ms_res), caller->name);
    
    while((ms_row = mysql_fetch_row(ms_res)))
    {
	send_out(":%s NOTICE %s :  %c  %d -> from %s -> Sent: %s",
		ms->name, caller->name, *ms_row[2] == 'n' ? '*' : ' ',
		++i, ms_row[0], ms_row[1]);
    }

    mysql_free_result(ms_res);

    return;
}

/* parv[0] -> memo id
 */
void ms_read(User *caller, gint parc, gchar **parv)
{
    gint num;
    time_t t;
    guint send_id, rcpt_id;
    
    if(parc < 1)
    {
	send_out(":%s NOTICE %s :Not enough parameters",
		ms->name, caller->name);
	return;
    }

    if(!(caller->mode & UMODE_r))
    {
	send_out(":%s NOTICE %s :You must identify to a registered nickname before attempting to read memos",
		ms->name, caller->name);
	return;
    }

    num = atoi(parv[0]);
    if(num <= 0)
    {
	send_out(":%s NOTICE %s :Invalid memo number (%s)",
		ms->name, caller->name, parv[0]);
	return;
    }

    sql_query_assert(&ms_res,
	    "SELECT `memo_sender`.`name`, DATE_FORMAT(`memo`.`date`, '%%b %%e %%Y - %%T'), `memo`.`text`, "
	    	"`memo`.`send_id`, `memo`.`rcpt_id`, `memo`.`read`, `memo`.`id` "
	    "FROM `memo`, `nicks` `memo_sender`, `nicks` `memo_rcpt` "
	    "WHERE ( `memo`.`rcpt_id` = `memo_rcpt`.`id` AND `memo_rcpt`.`name` = '%s' ) AND"
	    	"( `memo`.`send_id` = `memo_sender`.`id` )"
	    "ORDER BY `memo`.`date` "
	    "LIMIT %d,1 ",
	    caller->name, num - 1);

    if(!mysql_num_rows(ms_res))
    {
	mysql_free_result(ms_res);
	
	send_out(":%s NOTICE %s :Memo number %d does not exist !",
		ms->name, caller->name, num);
	return;
    }

    ms_row = mysql_fetch_row(ms_res);

    send_out(":%s NOTICE %s :Memo number %d from %s. To delete, type /ms DEL %d",
	    ms->name, caller->name, num, ms_row[0], num);
    send_out(":%s NOTICE %s :Sent: %s",
	    ms->name, caller->name, ms_row[1]);
    send_out(":%s NOTICE %s :Message: %s",
	    ms->name, caller->name, ms_row[2]);

    send_id = atoi(ms_row[3]);
    rcpt_id = atoi(ms_row[4]);

    if(*ms_row[5] == 'n')
    {
	sql_query_assert(NULL,
		"UPDATE `memo` SET `read` = 'y', `date` = FROM_UNIXTIME('%ld') "
		"WHERE `id` = '%s'",
		t, ms_row[6]);
    }

    return;
}

/* parv[0] -> rcpt
 * parv[1] -> text
 */
void ms_send(User *caller, gint parc, gchar **parv)
{
    guint send_id, rcpt_id;
    User *u;
    
    if(parc < 2)
    {
	send_out(":%s NOTICE %s :Not enough parameters",
		ms->name, caller->name);
	return;
    }

    if(!(caller->mode & UMODE_r))
    {
	send_out(":%s NOTICE %s :You must identify to a registered nickname before attempting to send memos",
		ms->name, caller->name);
	return;
    }

    sql_query_assert(&ms_res,
	    "SELECT `src_nick`.`id`, `dst_nick`.`id` "
	    "FROM `nicks` AS `src_nick` "
	    "LEFT JOIN `nicks` AS `dst_nick` ON `dst_nick`.`name` = '%s' "
	    "WHERE `src_nick`.`name` = '%s'",
	    parv[0], caller->name);

    ms_row = mysql_fetch_row(ms_res);
    
    if(!ms_row[1])
    {
	mysql_free_result(ms_res);
	send_out(":%s NOTICE %s :Nick %s is not registered !",
		ms->name, caller->name, parv[0]);
	return;
    }

    send_id = atoi(ms_row[0]);
    rcpt_id = atoi(ms_row[1]);

    mysql_free_result(ms_res);

    sql_query_assert(NULL,
	    "INSERT INTO `memo` VALUES(0, '%d', '%d', FROM_UNIXTIME('%ld'), '%s', 'n')",
	    send_id, rcpt_id, time(NULL), parv[1]);

    send_out(":%s NOTICE %s :Memo sent to %s",
	    ms->name, caller->name, parv[0]);

    if((u = _TBL(user).get(parv[0])) && u->mode & UMODE_r)
    {
	memo_notify(u, 1);
    }

    return;
}

/* parv[0] -> channel
 * parv[1] -> all|vop|aop|sop|cf
 * parv[2] -> message
 */
void ms_sendto(User *caller, gint parc, gchar **parv)
{
    guint send_id, chan_id = 0, founder_id = 0;
    gint id;
    gchar founder[NICKLEN + 1];
    static gchar memo[BUFSIZE];
    User *u;

    const struct ms_sendto_rcpt
    {
	gchar *table;
	gchar *desc;
    } ms_sendto_rcpts[] = {
	{"cfounder", "(CF+)"},
	{"sop", "(SOP+)"},
	{"aop", "(AOP+)"},
	{"vop", "(VOP+)"}
    };

    if(parc < 3)
    {
	send_out(":%s NOTICE %s :Not enough parameters",
		ms->name, caller->name);
	return;
    }

    if(!(caller->mode & UMODE_r))
    {
	send_out(":%s NOTICE %s :You must identify to a registered nickname before attempting to send memos",
		ms->name, caller->name);
	return;
    }

    my_strtoupper(parv[1]);
    if((id = ms_get_sendto_id(parv[1])) < 0)
    {
	send_out(":%s NOTICE %s :Invalid destination specification",
		ms->name, caller->name);
	return;
    }

    if(!ms_get_chan_and_founder_id(parv[0], &chan_id, &founder_id))
    {
	send_out(":%s NOTICE %s :Channel %s is not registered !",
		ms->name, caller->name, parv[0]);
	return;
    }

    sql_query_assert(&ms_res,
	    "SELECT `id`, `name` FROM `nicks` WHERE `name` = '%s'",
	    caller->name);

    ms_row = mysql_fetch_row(ms_res);
    send_id = atoi(ms_row[0]);
    strcpy(founder, ms_row[1]);
    mysql_free_result(ms_res);

    g_snprintf(memo, sizeof(memo), "%s %s %s",
	    parv[0], ms_sendto_rcpts[id].desc, parv[2]);

    /* send memo to channel founder
     */
    sql_query_assert(NULL,
	    "INSERT INTO `memo` VALUES(0, '%d', '%d', FROM_UNIXTIME('%ld'), '%s', 'n')",
	    send_id, founder_id, time(NULL), memo);

    if((u = _TBL(user).get(founder)) && u->mode & UMODE_r)
    {
	memo_notify(u, 1);
    }

    for(; id >= 0; id--)
    {
	gchar *table = ms_sendto_rcpts[id].table;

	sql_query_assert(&ms_res,
		"SELECT `%s`.`nick_id`, `nicks`.`name` "
		"FROM `%s` LEFT JOIN `nicks` ON `%s`.`nick_id` = `nicks`.`id`"
		"WHERE `%s`.`flags` = 'nick' AND `%s`.`chan_id` = '%d'",
		table, table, table, table, table, chan_id);

	if(!mysql_num_rows(ms_res))
	{
	    /* empty list
	     */
	    continue;
	}

	while((ms_row = mysql_fetch_row(ms_res)))
	{
	    User *u;

	    sql_query_assert(NULL,
		    "INSERT INTO `memo` VALUES(0, '%d', '%s', FROM_UNIXTIME('%ld'), '%s', 'n')",
		    send_id, *ms_row, time(NULL), memo);

	    if((u = _TBL(user).get(ms_row[1])))
	    {
		memo_notify(u, 1);
	    }
	}

	mysql_free_result(ms_res);
    }
}

/* parv[0] -> memo id to delete
 */
void ms_del(User *caller, gint parc, gchar **parv)
{
    guint id;
    gushort num;
    
    if(parc < 1)
    {
	send_out(":%s NOTICE %s :Not enough parameters",
		ms->name, caller->name);
	return;
    }

    if(!(caller->mode & UMODE_r))
    {
	send_out(":%s NOTICE %s :You must identify to a registered nickname before attempting to delete memos",
		ms->name, caller->name);
	return;
    }

    num = atoi(parv[0]);
    
    if(!num)
    {
	send_out(":%s NOTICE %s :Invalid memo number (%s)",
		ms->name, caller->name, parv[0]);
	return;
    }

    sql_query_assert(&ms_res,
	    "SELECT `memo`.`id` "
	    "FROM `memo`, `nicks` `memo_rcpt` "
	    "WHERE `memo`.`rcpt_id` = `memo_rcpt`.`id` AND `memo_rcpt`.`name` = '%s' "
	    "ORDER BY `memo`.`date` "
	    "LIMIT %d,1",
	    caller->name, num - 1);

    if(mysql_num_rows(ms_res))
    {
	ms_row = mysql_fetch_row(ms_res);

	id = atoi(ms_row[0]);

	sql_query_assert(NULL,
		"DELETE FROM `memo` "
		"WHERE `memo`.`id` = '%d'", id);

	send_out(":%s NOTICE %s :Memo number %d deleted",
		ms->name, caller->name, num);
    }
    else
    {
	send_out(":%s NOTICE %s :Memo number %d does not exist !",
		ms->name, caller->name, num);
    }

    mysql_free_result(ms_res);

    return;
}

static gboolean ms_memo_check(User *u)
{
    gushort count;

    sql_query_assert_ret(FALSE, &ms_res,
	    "SELECT COUNT(*) "
	    "FROM `memo`, `nicks` `memo_rcpt` "
	    "WHERE ( `memo`.`rcpt_id` = `memo_rcpt`.`id` AND `memo_rcpt`.`name` = '%s' ) AND "
	    "`memo`.`read` = 'n'",
	    u->name);

    ms_row = mysql_fetch_row(ms_res);

    count = atoi(*ms_row);

    if(count)
    {
	memo_notify(u, count);
    }

    return TRUE;
}

static G_INLINE_FUNC void memo_notify(User *u, gushort count)
{
    send_out(":%s NOTICE %s :You have %d new memos",
	    ms->name, u->name, count);
    send_out(":%s NOTICE %s :Type /ms LIST to list them and /ms READ <number> to read them",
	    ms->name, u->name);

    return;
}

static G_INLINE_FUNC gboolean ms_get_chan_and_founder_id(const gchar *chname, guint *chan_id, guint *founder_id)
{
    sql_query_assert_ret(FALSE, &ms_res, 
	    "SELECT `id`, `founder_id` FROM `chans` WHERE chans.name = '%s'",
	    chname);

    if(!mysql_num_rows(ms_res))
    {
	/* no .. */
	mysql_free_result(ms_res);
	return FALSE;
    }

    /* yes .. save the chan id for faster processing */
    ms_row = mysql_fetch_row(ms_res);
    *chan_id = atoi(ms_row[0]);
    *founder_id = atoi(ms_row[1]);

    mysql_free_result(ms_res);

    return TRUE;
}

/* EOF
 */
