/*
 * Copyright 2002 Barnaba Marcello <vjt@azzurra.org>
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2a. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 2b. Redistribution in binary form requires specific prior written
 *     authorization of the maintainer.
 * 
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *    This product includes software developed by Barnaba Marcello.
 * 
 * 4. The names of the maintainer, developers and contributors may not be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE MAINTAINER, DEVELOPERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * THE DEVELOPERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/* $Id: nickserv-validate-email.c,v 1.3 2003/03/23 18:10:14 vjt Exp $ */

#include "main.h"
#include "nickserv.h"
#include "ns-val-email.h"
#include <regex.h>

static regex_t re[3];
static gchar *re_txt[] = {
    "(@.*@)|(\\.\\.)|(@\\.)|(\\.@)|(^\\.)", /* invalid email (multiple '@') */
    "^.+\\@(\\[?)[-_a-zA-Z0-9\\.]+\\.([a-zA-Z]{2,3}|[0-9]{1,3})(\\]?)$", /* valid domainname */
    "^[_a-zA-Z0-9\\.\\-]+$", /* valid user/domain part */
};
static const gint re_flags = REG_EXTENDED|REG_ICASE|REG_NOSUB;
static gchar regerror_buf[BUFSIZE];

void ns_init_validate_email(void)
{
    gint err, i;
    
    for(i = 0; i < 3; i++)
    {
	if((err = regcomp(&re[i], re_txt[i], re_flags)))
	{
	    regerror(err, &re[0], regerror_buf, sizeof(regerror_buf));
	    g_critical_syslog("Regex compilation error [%s]: %s", re_txt[i], regerror_buf);
	    return;
	}
    }
}

void ns_cleanup_validate_email(void)
{
    gint i;

    for(i = 0; i < 3; i++)
    {
	regfree(&re[i]);
    }

    return;
}

gboolean ns_validate_email(gchar *email)
{
    gchar *user, *domain;
    gchar mailbuf[EMAILLEN + 1];

    g_strlcpy(mailbuf, email, sizeof(mailbuf));

    /* check for invalid mail:
     * if(match(multiple @ characters) OR !match(valid_domainname))
     *     return FALSE;
     */
    if(regexec(&re[0], mailbuf, 0, NULL, 0) == 0 ||
	    regexec(&re[1], mailbuf, 0, NULL, 0) == REG_NOMATCH)
    {
	return FALSE;
    }

    user = mailbuf;
    domain = strchr(user, '@');
    *domain++ = '\0';

    if(regexec(&re[2], user, 0, NULL, 0) == REG_NOMATCH ||
	    regexec(&re[2], domain, 0, NULL, 0) == REG_NOMATCH)
    {
	return FALSE;
    }
    
    return TRUE;
}
