/* 
 * taken from bahamut/src/ircsprintf.c.
 *
 * no copyright into the original file, but bahamut is under the GPL2 license.
 * so, i think, also this file should be under that license.
 * author: unknown. if you are the author of the original code, drop me a note
 * at vjt@openssl.it, so i`ll update this comment. thanks.
 *
 * modified a bit:
 * 1) glibized
 * 2) re-indented
 * 3) added sql escaping
 * 4) removed WANT_SNPRINTF #ifdef ( we never use snprintf in sql code, for speed:
 *    length of sql queries is known, and is also known the max length of a command
 *    an user can issue ).
 * 5) removed snprintf-related functions
 * 6) removed call to sprintf() when we encounter an unknown type.
 * 7) removed copies of parameters as local variables, due to the removal of the
 *    sprintf() call.
 * 
 * -vjt
 */

/* $Id: sql-printf.c,v 1.4 2003/03/27 02:10:29 vjt Exp $ */

#include "sql-printf.h"
#include "log.h"

static gchar num[12] = {
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};
static gchar itoa_tab[10] = {
    '0', '1', '2', '3', '4', '5', '6', '7', '8', '9'
};
static gchar xtoa_tab[16] = {
    '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
    'a', 'b', 'c', 'd', 'e', 'f'
};

static gchar *nullstring = "NULL";

G_INLINE_FUNC gint sql_vsprintf(gchar *buf, const gchar *format, va_list ap) 
{
    register gchar *s = NULL;
    gulong i, u;
    gint len = 0;

    while(*format)
    {
	u = 0;
	switch(*format)
	{
	    case '%':
		format++;
		switch(*format)
		{
		    case '%':
			buf[len++] = '%';
			format++;
			break;

		    case 's': /* most popular ;) */
			s = va_arg(ap, gchar *);			

			if(s == NULL)
			    s = nullstring;

			while(*s)
			{
			    /* sql escaping here
			     */
			    switch(*s)
			    {
				case '\\':
				    buf[len++] = '\\';
				    buf[len++] = *s++;
				    break;
				case '\'':
				    buf[len++] = '\\';
				    buf[len++] = *s++;
				    break;
				default:
				    buf[len++] = *s++;
				    break;
			    }
			}

			format++;
			break;
		    case 'u':
			format--;
			/* falls through and is caught below */
		    case 'l':
			if (*(format+1) == 'u')
			{
			    u = 1;
			    format++;
			}
			else if (*(format+1) == 'd')
			{
			    u = 0;
			    format++;
			}
			else
			    u = 0;
			/* fallthrough */
		    case 'd':
		    case 'i':
			i = va_arg(ap, gulong);
			if(!u)
			{
			    if(i & 0x80000000)
			    {
				buf[len++] = '-'; /* it's negative.. */
				i = 0x80000000 - (i & ~0x80000000);
			    }
			    s = &num[11];

			    do
			    {
				*--s = itoa_tab[i % 10];
				i /= 10;
			    } while(i != 0);

			    while(*s)
			    {
				buf[len++] = *s++;
			    }

			    format++;
			    break;
			}
		    case 'n':
			/* oo, sneaky...it really is just a long, though! */
		    case 'x':
		    case 'X':
			i = va_arg(ap, glong);
			buf[len++] = '0';
			buf[len++] = 'x';
			s = &num[11];

			do
			{
			    *--s = xtoa_tab[i % 16];
			    i /= 16;
			} while(i != 0);

			while(*s)
			{
			    buf[len++]=*s++;
			}

			format++;
			break;

		    case 'c':
			buf[len++] = (gchar) va_arg(ap, gint);
		    format++;
		    break;
		default:
		    /* yick, unknown type...default to returning what our 
		       s[n]printf friend would */

		    /* no. we use only the types already defined: if we need another
		     * type, we`ll also need to implement that type here.
		     * -vjt
		     */
		    /* return vsprintf(str, pattern, vl); */
		    g_critical("Unknown type passed to sql_printf(): `%c'", *format);
		    return -1;
		}
		break;
	    default:
		buf[len++] = *format++;
		break;
	}
    }

    buf[len] = '\0';
    return len;
}

gint sql_sprintf(gchar *str, const gchar *format, ...)
{
    gint ret;
    va_list vl;
    
    va_start(vl, format);
    ret = sql_vsprintf(str, format, vl);
    va_end(vl);

    return ret;
}
